"use strict";
this.name        = "torustosun";
this.author      = "Norby";
this.copyright   = "2014 Norbert Nagy";
this.licence     = "CC BY-NC-SA 4.0";
this.description = "Reach the Sun faster.";
this.version     = "1.3";

//http://aegidian.org/bb/viewtopic.php?f=6&t=16704&p=223460#p223405
//-start at baseTorusMultiplier when you leave the masslock of planet,
//-increasing exponentially between the planet and sun up to 8x,
//-fall down to the base value before the sun,
//-limited to the max value if far from both the sun and planet.

//customizable properties
this.$TorusToSunMaxMultiplier = 8; //*32*400m/s (Asp) almost fit into the 5 digit NumericHUD
this.$TorusToSunTimeFw = 8; //time forwarding at max. multiplier (adjustment, not TAF)
this.$TorusToSunMessages = true; //show messages about the current speed (but no messages with CombatMFD regardless of this)

//internal properties, should not touch
this.$TorusToSunBonus = 1; //current speed multiplier
this.$TorusToSunDBonus = ""; //the lastly displayed speed multiplier
this.$TorusToSunFCB = null; //framecallback
this.$TorusToSunFwSec = 0; //part of a second during forwarding
this.$TorusToSunNearestPlanet = null; //store the nearest planet between rechecks
this.$TorusToSunPrevEnergy = 0; //previous energy value of player
this.$TorusToSunTimer = null; //Timer for Torus speedup
this.$TorusToSunWFP = null; //store worldScripts.farplanets

//world script events
this.startUp = function() {
	this.$TorusToSunWFP = worldScripts.farplanets;
	var n = worldScripts.numerichudv3;
//	if( n && n.version * 1 >= 3.24 ) this.$TorusToSunMessages = false;
}

this.shipWillDockWithStation = function() {
	this.$TorusToSun_StopTimer();
}

this.shipWillEnterWitchspace = function() {
	this.$TorusToSun_StopTimer();
}

this.shipWillExitWitchspace = function() {
	this.$TorusToSun_StartTimer(); //must start before Telescope FCBs!
}

this.shipWillLaunchFromStation = function() {
	this.$TorusToSun_StartTimer(); //must start before Telescope FCBs!
}

//TorusToSun functions
this.$TorusToSun_FCB = function( delta ) { //masslockcheck in this framecallback
	var b = this.$TorusToSunBonus;
	var ps = player.ship;
	if( !ps ) return; //player is died
//	log("TorusToSun", "speed: "+Math.round(ps.speed)+" velocity: "+Math.round(ps.velocity.magnitude())); //debug
	if( player.alertCondition > 1 || ps.speed < ps.maxSpeed * 31.9 //not in full torus
		|| this.$TorusToSunWFP && //FarPlanets OXP active
		( this.$TorusToSunWFP.$FarPlanetsSlowDown || this.$TorusToSunWFP.$FarPlanetsBonus > 1 ))
		 this.$TorusToSun_TorusSlowDown(); //do not apply TorusToSun
	if( !this.$TorusToSunSlowDown && b > 1 ) { //currently travel at bonus speeds
		var f = ps.vectorForward;
		var t = ps.maxSpeed * 32; //torus speed
		if( b == this.$TorusToSunMaxMultiplier) t = t * this.$TorusToSunTimeFw; //time forwarding
		ps.position = ps.position.add( f.multiply( delta * t * ( b - 1 ) ) ); //apply speed bonus
	}
}

this.$TorusToSun_NearestMass = function(skip) {
	var d = 100000000000000000000.0; //10^20m for sure
	var psp = player.ship.position;
	var nm = null;
	var p = system.planets;
	var len = p.length;
	for( var i = 0; i < len; i++ ) { //find the smallest distance
		if( p[i] != skip ) { //the second nearest if requested
			var s = psp.distanceTo( p[i].position );
			if( d > s ) {
				d = s;
				nm = p[i];
			}
		}
	}
	return( nm );
}

this.$TorusToSun_StartTimer = function() {
	var ws = worldScripts.torustosun;
	ws.$TorusToSunPrevEnergy = player.ship.maxEnergy;
	ws.$TorusToSunFwSec = 0; //part of a second during forwarding
	ws.$TorusToSunTimer = new Timer(ws, ws.$TorusToSun_Timed.bind(ws), 1, 0.25);
	if( !isValidFrameCallback( ws.$TorusToSunFCB ) )
		ws.$TelescopeVFCB = addFrameCallback( ws.$TorusToSun_FCB.bind(ws) );
}

this.$TorusToSun_StopTimer = function() {
	var ws = worldScripts.torustosun;
	if( ws.TorusToSunTimer ) {
		ws.$TorusToSunTimer.stop();
		delete ws.$TorusToSunTimer;
	}
	if( isValidFrameCallback( ws.$TorusToSunFCB ) )
		removeFrameCallback( ws.$TorusToSunFCB );
	this.$TorusToSun_TorusSlowDown(); //set bonus to 1
}

this.$TorusToSun_Timed = function() {
	var ps = player.ship;
	if( !ps || !ps.isValid ) return; //player died
	if( player.alertCondition == 1 && ps.speed > ps.maxSpeed * 31.9 ) {//Green Alert and Torus maxed
		this.$TorusToSun_TorusSpeedUp();
	}
}

this.$TorusToSun_TorusSlowDown = function() { //slowdown from extreme speed to normal or Torus speeds
	if( this.$TorusToSunBonus <= 1 ) return; //need to avoid block Injector speeds
	this.$TorusToSunPrevEnergy = player.ship.energy;
	this.$TorusToSunBonus = 1; //the main thing for slowdown
	this.$TorusToSunDBonus = "";
	this.$TorusToSunSlowDown = true; //flag to FCB
}

this.$TorusToSun_TorusSpeedUp = function() {
	var ps = player.ship;
	if( !ps || !ps.isValid || this.$TorusToSunWFP && //FarPlanets OXP active
		( this.$TorusToSunWFP.$FarPlanetsSlowDown || this.$TorusToSunWFP.$FarPlanetsBonus > 1 ))
		return;

	var bonus = 1;
	if( ps.equipmentStatus("EQ_TORUSTOSUN") == "EQUIPMENT_OK"
		&& system && system.planets && system.planets.length > 0
		&& !system.isInterstellarSpace && system.sun && system.sun.isValid
		&& !system.sun.hasGoneNova ) {
		this.$TorusToSunSlowDown = false;
	
		var co = 1000; //coefficient, larger value cause smaller speeds
		var mm = this.$TorusToSunMaxMultiplier;

		var sr = system.sun.radius;
		var pds = ps.position.distanceTo( system.sun.position ) - sr; //playerDistanceFromSun
		//check sun direction and apply gravity pullback, angle=0: aft, angle=PI: front
		var angle = ps.vectorForward.angleTo(ps.position.subtract(system.sun.position));
		sr = Math.min(sr, 200000); //max. 200km in the following formula
		//keep up the high speed much longer when approach the sun
		var min = pds / ( 10000 + ( 1 - angle / Math.PI ) * sr *
			Math.max( 0, 1 - pds / ( 10 * sr ) ) ); //no gravity pullback over 10 sun radius
		bonus = 1 + (mm - 1) * Math.min( 1, min * min / co ); //nonlinear calculation

		var nm = this.$TorusToSun_NearestMass();
		var nr = Math.max(nm.radius, 50000); //min. 50k to reduce speeds near small moons
		var pdp = ps.position.distanceTo( nm.position ) - 2 * nr; //playerDistanceFromPlanet
		min = pdp / nr;
		var b1 = 1 + (mm - 1) * Math.min( 1, min * min / co ); //nonlinear calculation
		bonus = Math.min(b1, bonus); //use the smaller bonus
		
		var nm2 = this.$TorusToSun_NearestMass(nm); //the second nearest mass
		if( nm2 ) {
			var nr2 = Math.max(nm2.radius, 50000); //min. 50k to reduce speeds near small moons
			var pdp2 = ps.position.distanceTo( nm2.position ) - 2 * nr2; //playerDistanceFromPlanet2
			min = pdp2 / nr2;
			var b2 = 1 + (mm - 1) * Math.min( 1, min * min / co ); //nonlinear calculation
			bonus = Math.min(b2, bonus); //use the smallest bonus
		}
//		log("TorusToSun", " bonus:"+bonus+" pdp:"+pdp+" pds:"+pds+" nm:"+nm); //debug
	}

	if( bonus > 1 ) { //show speed bonus, apply will happen in FCB
		var b = Math.round( bonus );
		if( bonus == mm ) { //use time forwarding at maximal bonus
			var delta = 0.25; //0.25sec timer
			//calculate the bonus time plus add the previous remainder
			var sec = delta * this.$TorusToSunTimeFw * (bonus - 1) + this.$TorusToSunFwSec;
//			log("TorusToSun", " bonus:"+bonus+" mm:"+mm+" sec:"+sec); //debug
			if( sec > 1 ) {
				var maxsec = 2592000; //core is not allow to add more sec in one step
				if( sec < maxsec ) clock.addSeconds( Math.floor( sec ) ); //add the integer part
				else {
					var sec2 = Math.floor( sec / maxsec );
					for(var i=0; i<sec2; i++) clock.addSeconds( maxsec );
					clock.addSeconds( sec - maxsec * sec2 );
				}
//				log("TorusToSun", "added "+sec+"s = "+(sec/86400)+" day"); //debug
				this.$TorusToSunFwSec = sec - Math.floor( sec ); //save the remainder
			} else this.$TorusToSunFwSec = sec;
			if( this.$TorusToSunPrevEnergy < ps.energy && ps.energy < ps.maxEnergy - 1 ) {
				//recharge speed up scaled with forwarding
				ps.energy +=  ( ps.energy - this.$TorusToSunPrevEnergy ) * sec / delta;
			}
		}
		if( this.$TorusToSunMessages && this.$TorusToSunDBonus != b
			&& ps.equipmentStatus("EQ_COMBATMFD") != "EQUIPMENT_OK") { //no mesages with MFD
			player.consoleMessage( "Torus " + b + "x", 5 );
		}
		this.$TorusToSunPrevEnergy = ps.energy;
		this.$TorusToSunBonus = bonus;
		this.$TorusToSunDBonus = b; //save last message to avoid repeat
	}  else this.$TorusToSun_TorusSlowDown(); //slowdown
}